/**
 * Copyright (c) 2005-2012 EBM WebSourcing, 2012-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task;

import java.util.ArrayList;
import java.util.List;

import org.ow2.petals.ant.AbstractJBIAntTask;

/**
 * Ant task : jbi-list-shared-libraries. This task prints information about all
 * shared libraries installed in JBI environment.
 * 
 * @author ddesjardins - EBM WebSourcing
 * @author Christophe Hamerling - EBM WebSourcing
 */
public class ListSharedLibrariesTask extends AbstractJBIAntTask {

    private static final String XMLNS = "http://java.sun.com/xml/ns/jbi/shared-library-info-list";

    /**
     * List the shared libraries that the component depends on. Without this
     * parameter, all the shared libraries in the JBI Environment are displayed.
     */
    private String componentName;

    /**
     * If supplied, only the named shared library is reported. If the shared library doesn’t exist, an empty
     * component-list report is given. Must be used with {@link #sharedLibraryVersion}.
     */
    private String sharedLibraryName;

    /**
     * List the Bindings that are dependent on the specified shared library. Without this parameter, no shared library
     * dependency will be verified. Must be used with {@link #sharedLibraryName}.
     */
    private String sharedLibraryVersion;

    /**
     * If supplied, set the given Ant property name value to the XML version of this listing task.
     */
    private String xmlOutput;

    /**
     * Creates a new instance of {@link ListSharedLibrariesTask}
     * 
     */
    public ListSharedLibrariesTask() {
        super();
    }

    @Override
    public void doTask() throws Exception {

        final String[][] sharedLibraryNames = this.getJMXClient().getInstallationServiceClient()
                .getInstalledSharedLibraries();

        final List<String[]> sharedLibraryList = new ArrayList<String[]>();

        for (final String[] sl : sharedLibraryNames) {
            final String slName = sl[0];
            final String slVersion = sl[1];

            // skip this SL if component name has been provided and is not equal
            // to the component dependencies to the current one
            if (componentName != null) {
                final String[] componentNames = this.getJMXClient().getInstallationServiceClient()
                        .getInstalledComponentsForSharedlibrary(slName, slVersion);

                boolean found = false;
                for (final String compName : componentNames) {
                    if (componentName.equals(compName)) {
                        found = true;
                        break;
                    }
                }
                if (!found) {
                    continue;
                }
            }

            // skip this SL if name has been provided and is not equal to
            // the current one
            if (sharedLibraryName != null) {
                if (!sharedLibraryName.equals(slName)) {
                    continue;
                }
            }

            // skip this SL if version has been provided and is not equal to
            // the current one
            if (sharedLibraryVersion != null) {
                if (!sharedLibraryVersion.equals(slVersion)) {
                    continue;
                }
            }

            sharedLibraryList.add(sl);
        }

        final StringBuffer output = new StringBuffer();
        final StringBuffer outputXML = new StringBuffer();
        output.append("#######################################################\n");
        output.append("#                    Shared Libraries                 #\n");
        output.append("#######################################################\n");
        outputXML.append("<?xml version='1.0' encoding='UTF-8'?>\n");
        outputXML.append("<shared-library-info-list ");
        outputXML.append("xmlns='" + XMLNS + "' version='1.0'>\n");

        for (final String[] sl : sharedLibraryList) {
            // TODO get the state and the description
            final String description = "TODO";
            final String slName = sl[0];
            final String slversion = sl[1];

            output.append("Name        : " + slName + "\n");
            output.append("Version     : " + slversion + "\n");
            output.append("Description : " + description + "\n\n");

            outputXML.append("\t<shared-library-info name='");
            outputXML.append(slName);
            outputXML.append("' version='");
            outputXML.append(slversion);
            outputXML.append("'>\n\t\t<description>");
            outputXML.append(description);
            outputXML.append("</description>\n");
            outputXML.append("\t</shared-library-info>");
        }
        outputXML.append("</shared-library-info-list>");

        if (xmlOutput == null) {
            log(output.toString());

        } else {
            getProject().setNewProperty(xmlOutput, outputXML.toString());
        }
    }

    /**
     * @param componentName
     *            the componentName to set
     */
    public void setComponentName(String componentName) {
        this.componentName = componentName;
    }

    /**
     * @param sharedLibraryName
     *            the sharedLibraryName to set
     */
    public void setSharedLibraryName(String sharedLibraryName) {
        this.sharedLibraryName = sharedLibraryName;
    }

    /**
     * @param sharedLibraryVersion
     *            the sharedLibraryVersion to set
     */
    public void setSharedLibraryVersion(final String sharedLibraryVersion) {
        this.sharedLibraryVersion = sharedLibraryVersion;
    }

    /**
     * @param xmlOutput
     *            the xmlOutput to set
     */
    public void setXmlOutput(String xmlOutput) {
        this.xmlOutput = xmlOutput;
    }

}
