/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit.assertion.flowinstance;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import java.util.Set;

import org.apache.tools.ant.Project;
import org.ow2.petals.ant.task.monit.AbstractMonitFlowsAntTaskRequiringRefId;
import org.ow2.petals.ant.task.monit.FlowStepInfo;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AssertAttribute;
import org.ow2.petals.ant.task.monit.assertion.flowstep.AssertFlowStep;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildAssertFlowStepEndException;
import org.ow2.petals.ant.task.monit.assertion.flowstep.exception.BuildFlowInstanceIdInvalidException;
import org.ow2.petals.ant.task.monit.exception.BuildFlowStepIdxInvalidException;
import org.ow2.petals.ant.task.monit.exception.BuildFlowStepIdxMissingException;
import org.ow2.petals.ant.task.monit.exception.BuildPropertyMissingException;
import org.ow2.petals.log.parser.api.model.Flow;
import org.ow2.petals.log.parser.api.model.FlowStep;

/**
 * A Ant task to create asserts on a flow instance read from MONIT traces.
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class AssertFlowInstanceTask extends AbstractMonitFlowsAntTaskRequiringRefId {

    /**
     * The identifier of the flow instance on which assertions must be done
     */
    private String flowInstanceId;

    /**
     * Assertions to execute on flow steps
     */
    private final List<AssertFlowStep> assertionsOnFlowSteps = new ArrayList<>();

    /**
     * @param flowInstanceId
     */
    public void setFlowInstanceId(final String flowInstanceId) {
        this.flowInstanceId = flowInstanceId;
    }

    public void addAssertFlowStep(final AssertFlowStep assertionsOnAFlowStep) {
        this.assertionsOnFlowSteps.add(assertionsOnAFlowStep);
    }

    @Override
    public final void doTaskWithFlows(final Set<Flow> flows) throws Exception {

        Collections.sort(this.assertionsOnFlowSteps, new Comparator<AssertFlowStep>() {

            @Override
            public int compare(final AssertFlowStep o1, final AssertFlowStep o2) {
                if (o1.getFlowStepIdx() == null) {
                    throw new BuildFlowStepIdxMissingException(AssertFlowInstanceTask.this.getLocation());
                } else if (o2.getFlowStepIdx() == null) {
                    throw new BuildFlowStepIdxMissingException(AssertFlowInstanceTask.this.getLocation());
                } else {
                    return o1.getFlowStepIdx().compareTo(o2.getFlowStepIdx());
                }
            }
        });

        boolean flowInstanceFound = false;
        for (final Flow flow : flows) {
            if (flow.getFlowId().equals(this.flowInstanceId)) {
                // It's the expected flow instance
                flowInstanceFound = true;

                final List<FlowStep> stepsOfFlowInstance = this.getAllFlowSteps(flow.getRoot());
                this.getProject().log("Running assertion on the following flow instance: ", Project.MSG_DEBUG);
                for (final FlowStep flowStep : stepsOfFlowInstance) {
                    this.getProject().log(flowStep.toString(), Project.MSG_DEBUG);
                }

                for (final AssertFlowStep assertFlowStep : this.assertionsOnFlowSteps) {
                    final Integer flowStepIdx = assertFlowStep.getFlowStepIdx();
                    assert flowStepIdx != null;
                    this.getProject().log("Asserting flow step #" + flowStepIdx, Project.MSG_VERBOSE);

                    final FlowStep selectedFlowStep;
                    try {
                        selectedFlowStep = stepsOfFlowInstance.get(flowStepIdx);
                    } catch (final IndexOutOfBoundsException e) {
                        throw new BuildFlowStepIdxInvalidException(this.getLocation(), e);
                    }

                    if ((!assertFlowStep.getEndsWithError() && selectedFlowStep.isFailure())
                            || (assertFlowStep.getEndsWithError() && !selectedFlowStep.isFailure())) {
                        throw new BuildAssertFlowStepEndException(assertFlowStep.getEndsWithError(),
                                this.getLocation());
                    }

                    for (final AssertAttribute attributeAssertion : assertFlowStep.getAttributeAssertions()) {
                        attributeAssertion.doAssert(selectedFlowStep, this.getProject(), this.getLocation());
                    }
                    for (final FlowStepInfo flowStepInfo : assertFlowStep.getFlowStepInfos()) {
                        final String property = flowStepInfo.getProperty();
                        if (property == null) {
                            throw new BuildPropertyMissingException(this.getLocation());
                        } else {
                            this.getProject().setProperty(property,
                                    selectedFlowStep.getBeginProperties().get(flowStepInfo.getAttribute()));
                        }
                    }
                }

                break;
            }
        }

        if (!flowInstanceFound) {
            throw new BuildFlowInstanceIdInvalidException(this.getLocation());
        }

    }

}
