/**
 * Copyright (c) 2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.ant.task.monit;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.fail;

import java.net.URISyntaxException;
import java.util.Collection;

import org.apache.tools.ant.Project;
import org.junit.Test;
import org.ow2.petals.ant.task.monit.exception.BuildIdEmptyException;
import org.ow2.petals.ant.task.monit.exception.BuildIdMissingException;
import org.ow2.petals.ant.task.monit.exception.BuildRefIdEmptyException;
import org.ow2.petals.ant.task.monit.exception.BuildRefIdInvalidException;
import org.ow2.petals.ant.task.monit.exception.BuildRefIdMissingException;

/**
 * Unit tests of {@link BuildFlowTreesTask}
 * 
 * @author Christophe DENEUX - Linagora
 *
 */
public class BuildFlowCorrelationTreeTaskTest extends AbstractMonitTraceTest {

    /**
     * <p>
     * Check the flow step information extraction from MONIT traces generated from a BC Gateway usecase, on nominal
     * conditions when criterias (without row number) select one flow step.
     * </p>
     * <p>
     * Expected trees are as following: {@code "initialdomain" --- "intermediate domain" ---- "target domain"}
     * </p>
     */
    @Test
    public void bcGateway() throws URISyntaxException {

        final Project project = this.loadMonitTraces("monit/flow-monitoring-correlated/gw-container-0",
                "monit/flow-monitoring-correlated/gw-container-3.0",
                "monit/flow-monitoring-correlated/gw-container-3.1",
                "monit/flow-monitoring-correlated/gw-container-4.0",
                "monit/flow-monitoring-correlated/gw-container-4.1");

        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setRefid(REF_ID);
        task.setId(REF_TREE_ID);
        task.setProject(project);

        task.execute();

        final Collection<FlowTreeNode> flowTrees = (Collection<FlowTreeNode>) project.getReference(REF_TREE_ID);

        // Assert number of flow trees
        assertEquals(4, flowTrees.size());
    }

    /**
     * <p>
     * Check the flow step information extraction from MONIT traces generated from a SE Activiti usecase, on nominal
     * conditions when criterias (without row number) select one flow step.
     * </p>
     * <p>
     * Expected trees are as following: <code>
     * 
     * createVacationRequest ----\
     *                            \
     *                             |--------- Activiti process
     *                            /
     * userTask 1 ---------------/
     * </code>
     * </p>
     */
    @Test
    public void seActiviti() throws URISyntaxException {

        final Project project = this.loadMonitTraces("monit/flow-monitoring-correlated/activiti");

        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setRefid(REF_ID);
        task.setId(REF_TREE_ID);
        task.setProject(project);

        task.execute();

        final Collection<FlowTreeNode> flowTrees = (Collection<FlowTreeNode>) project.getReference(REF_TREE_ID);

        // Assert number of flow trees
        assertEquals(3, flowTrees.size());

        for (final FlowTreeNode flowTreeNode : flowTrees) {
            if ("15f132a0-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the process instance creation
                assertEquals(0, flowTreeNode.getParents().size());
                assertEquals(1, flowTreeNode.getFlowTreeNodes().size());
            } else if ("160eccc0-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the process instance execution
                assertEquals(2, flowTreeNode.getParents().size());
                assertEquals(0, flowTreeNode.getFlowTreeNodes().size());
            } else if ("1a319b70-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the user task validation
                assertEquals(0, flowTreeNode.getParents().size());
                assertEquals(1, flowTreeNode.getFlowTreeNodes().size());
            } else {
                fail("Unexpectes flow instance id: " + flowTreeNode.getFlow().getFlowId());
            }
        }
    }

    /**
     * Check the processing of Ant task {@link BuildFlowTreesTask} when the parameter 'refid' is missing.
     */
    @Test(expected = BuildRefIdMissingException.class)
    public void noRefId() throws URISyntaxException {

        final BuildFlowTreesTask task = new BuildFlowTreesTask();

        task.execute();
    }

    /**
     * Check the processing of Ant task {@link BuildFlowTreesTask} when the parameter 'refid' is empty.
     */
    @Test(expected = BuildRefIdEmptyException.class)
    public void emptyRefId() throws URISyntaxException {

        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setRefid("");

        task.execute();
    }

    /**
     * Check the processing of Ant task {@link BuildFlowTreesTask} when the parameter 'refid' is unknown (not
     * previously loaded).
     */
    @Test(expected = BuildRefIdInvalidException.class)
    public void unexistingRefId() throws URISyntaxException {

        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setRefid(REF_ID);
        task.setProject(new Project());

        task.execute();
    }

    /**
     * Check the processing of Ant task {@link BuildFlowTreesTask} when the parameter 'id' is missing.
     */
    @Test(expected = BuildIdMissingException.class)
    public void noId() throws URISyntaxException {

        final Project project = this.loadMonitTraces();
        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setRefid(REF_ID);
        task.setProject(project);

        task.execute();
    }

    /**
     * Check the processing of Ant task {@link BuildFlowTreesTask} when the parameter 'id' is empty.
     */
    @Test(expected = BuildIdEmptyException.class)
    public void emptyId() throws URISyntaxException {

        final Project project = this.loadMonitTraces();
        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setId("");
        task.setRefid(REF_ID);
        task.setProject(project);

        task.execute();
    }

    /**
     * <p>
     * Check the flow step information extraction from MONIT traces generated from a SE Activiti usecase, on a
     * <b>running process instance (that is not finished, a service task remains to be executed)</b> when criterias
     * (without row number) select one flow step.
     * </p>
     * <p>
     * Expected trees are as following: <code>
     * 
     * createVacationRequest ----\
     *                            \
     *                             |--------- Activiti process
     *                            /
     * userTask 1 ---------------/
     * </code>
     * </p>
     */
    @Test
    public void seActiviti_processNotEnded_serviceTaskRemains() throws URISyntaxException {

        final Project project = this
                .loadMonitTraces("monit/flow-monitoring-correlated/activiti-process-not-finished-servicetask-remains");

        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setRefid(REF_ID);
        task.setId(REF_TREE_ID);
        task.setProject(project);

        task.execute();

        final Collection<FlowTreeNode> flowTrees = (Collection<FlowTreeNode>) project.getReference(REF_TREE_ID);

        // Assert number of flow trees
        assertEquals(3, flowTrees.size());

        for (final FlowTreeNode flowTreeNode : flowTrees) {
            if ("15f132a0-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the process instance creation
                assertEquals(0, flowTreeNode.getParents().size());
                assertEquals(1, flowTreeNode.getFlowTreeNodes().size());
            } else if ("160eccc0-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the process instance execution
                assertEquals(2, flowTreeNode.getParents().size());
                assertEquals(0, flowTreeNode.getFlowTreeNodes().size());
            } else if ("1a319b70-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the user task validation
                assertEquals(0, flowTreeNode.getParents().size());
                assertEquals(1, flowTreeNode.getFlowTreeNodes().size());
            } else {
                fail("Unexpectes flow instance id: " + flowTreeNode.getFlow().getFlowId());
            }
        }
    }

    /**
     * <p>
     * Check the flow step information extraction from MONIT traces generated from a SE Activiti usecase, on a
     * <b>running process instance (that is not finished, the user task completion is in progress, not finished)</b>
     * when criterias (without row number) select one flow step.
     * </p>
     * <p>
     * Expected trees are as following: <code>
     * 
     * createVacationRequest ----\
     *                            \
     *                             |--------- Activiti process
     * </code>
     * </p>
     */
    @Test
    public void seActiviti_processNotEnded_blockedOnUsertask() throws URISyntaxException {

        final Project project = this
                .loadMonitTraces("monit/flow-monitoring-correlated/activiti-process-not-finished-blocked-on-usertask");

        final BuildFlowTreesTask task = new BuildFlowTreesTask();
        task.setRefid(REF_ID);
        task.setId(REF_TREE_ID);
        task.setProject(project);

        task.execute();

        final Collection<FlowTreeNode> flowTrees = (Collection<FlowTreeNode>) project.getReference(REF_TREE_ID);

        // Assert number of flow trees
        assertEquals(3, flowTrees.size());

        for (final FlowTreeNode flowTreeNode : flowTrees) {
            if ("15f132a0-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the process instance creation
                assertEquals(0, flowTreeNode.getParents().size());
                assertEquals(1, flowTreeNode.getFlowTreeNodes().size());
            } else if ("160eccc0-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the process instance execution
                assertEquals(1, flowTreeNode.getParents().size());
                assertEquals(0, flowTreeNode.getFlowTreeNodes().size());
            } else if ("1a319b70-0575-11e6-83d8-024282c01af2".equals(flowTreeNode.getFlow().getFlowId())) {
                // Flow associated to the user task validation
                assertEquals(0, flowTreeNode.getParents().size());
                assertEquals(0, flowTreeNode.getFlowTreeNodes().size());
            } else {
                fail("Unexpectes flow instance id: " + flowTreeNode.getFlow().getFlowId());
            }
        }
    }

}
